# -*- coding: UTF-8 -*-

import MySQLdb
import nes_common.logger as logger
from nes_common import rest_client
from nes_common.executor import Executor, DataIntegrityError
from nes_common.sql_operation import _Insert


class Context(object):
    """ Контекст выполнения SQL запросов """

    def __init__(self, cursor, log, shopCode, loadDictStatusSender=None):
        """ Контекст выполнения SQL запросов
        Args:
            cursor (Cursor): Соединения с базой данных
            shopCode (str): Код магазина, работа с данными которого предполагается
            loadDictStatusSender (LoadDictStatusSender): Отправщик статуса справочников
        """
        self.cursor = cursor
        self.log = log
        self.emptyTables = {}
        self.shopCode = shopCode
        self.loadDictStatusSender = loadDictStatusSender
        self.tables = []

    def tableName(self, table):
        """ Получить окончательное название таблицы и отправить оповещение о начале работы с ней
        Args:
            table (str): Базовое название таблицы
        Returns:
            str: Окончательное название таблицы
        """

        if not self.tables.__contains__(table) and self.loadDictStatusSender:
            self.tables.append(table)
            self.log.info('table = %s' % table)
            self.loadDictStatusSender.sendDictState('Start load dict to db',
                    'Загрузка справочника в БД кассового сервера', rest_client.STATUS_DURING, tableName=table)
        return table

    def finalize(self):
        """ Выполнить завершающие операции
        Returns:
            bool: Успешно или нет
        """
        return True


class Exec(Executor):
    migrateVersions = {}

    def __init__(self, host, port, user, password, database, shopCode, loadDictStatusSender=None):
        self.database = database
        self.clearCache = False
        self.log = logger.getLogger("Shop", shopCode)
        if not password or password == '':
            connection = MySQLdb.connect(host=host, port=port, user=user, db=database)
            self.connectionAll = MySQLdb.connect(host=host, port=port, user=user, db=database.replace('dictionaries', "artixcs"))
        else:
            connection = MySQLdb.connect(host=host, port=port, user=user, passwd=password, db=database)
            self.connectionAll = MySQLdb.connect(host=host, port=port, user=user, passwd=password, db=database.replace('dictionaries', "artixcs"))
        self.connectionAll.autocommit(True)
        connection.set_character_set('utf8')
        connection.ping(True)
        context = Context(connection.cursor(), self.log, shopCode, loadDictStatusSender)
        Executor.__init__(self, shopCode, connection, context)

    def execute(self, sql):
        if self.clearCache and isinstance(sql, _Insert):
            self.log.debug('Old cache: %s' % sql.cacheOddColumns)
            sql.cacheOddColumns.clear()
            self.log.debug('New cache: %s' % sql.cacheOddColumns)
            sql.cacheOwnerStoreIdPresented.clear()
            self.clearCache = False
        Executor.execute(self, sql)

    def startUpload(self, messageId):
        Executor.startUpload(self, messageId)
        try:
            cur = self.connectionAll.cursor()
            cur.execute("SELECT version FROM migrate_version WHERE repository_id LIKE 'artixcs%';")
            row = cur.fetchone()
            currentMigrateVersion = Exec.migrateVersions.get(self.database)
            dbVersion = row[0] if row is not None else currentMigrateVersion
            if currentMigrateVersion != dbVersion and currentMigrateVersion is not None:
                self.log.info('migrateVersion is changed from %s to %s. Clear cache with tables columns' % (currentMigrateVersion, dbVersion))
                self.clearCache = True
            Exec.migrateVersions[self.database] = dbVersion
        except Exception as e:
            self.log.warning("Can't get migrate version. %s" % e)
        finally:
            cur.close()
            self.connectionAll.close()

    def flushTables(self):
        # Для КС таблицы флушить не нужно(т.к. никаких хитрых операций с файлами БД на ФС не производится в режиме КС)
        pass
