# -*- coding: UTF-8 -*-

import atexit
import json
import nes_common.logger as logger
import optparse
import os
import signal
import sys
import tempfile
import threading
import time
import traceback
from artix.common import servicelock
from artix.filequeue.filequeuemanager import FileQueueManager
from bottle import route, run, response, request, Bottle
from nes_common import rest_client
from nes_common.basictaskrunner import BasicTaskRunner
from nes_common.mproctaskrunner import MprocTaskRunner
from nes_common.config import config
from transaction import Transaction

prefixQueueName = 'artix.loaddict.'
INTERNAL_SERVER_ERROR = 500


def generateDict(log, shopId, dictBackOfficeIds, commandOwner, generateList, defaultStatusServer):
    if dictBackOfficeIds or generateList:
        log.info('Запускаем генерацию справоников на КС')
        rest_client.generateDict(shopId, dictBackOfficeIds, commandOwner, generateList, defaultStatusServer)


def receiveMessages(shopDest):
    shop = {'shopId': shopDest.split('.')[0]}
    shop['clusterId'] = shopDest.replace(shop['shopId'] + '.', '') if shop['shopId'] != shopDest else ''
    log = logger.getLogger("Shop", shop['shopId'])
    queueName = prefixQueueName + shopDest

    dictBackOfficeIds = set()
    generateList = {}
    commandOwner = ''
    try:
        fileQueueManager = FileQueueManager(queueRoot=Transaction.fileQueueRoot, log=log, location=('cash' if config.isCash else 'cs'))
        while True:
            transactionId = fileQueueManager.getCompleteTransactionId(queueName)
            if not transactionId:
                break
            transaction = Transaction(shop, log, queueName, transactionId, config.isCash)
            if not transaction.handle():
                break
            dictBackOfficeIds.update(transaction.dictBackOfficeIds)
            generateList.update(transaction.generateList)
            commandOwner = transaction.commandOwner
    except Exception as e:
        log.error('Ошибка: %s' % str(e))
        return False

    generateDict(log, shop['shopId'], dictBackOfficeIds, commandOwner, generateList, Transaction.defaultStatusServer)
    return True


class RestReceiver(Bottle):
    def __init__(self, taskRunner, receiveMethod):
        super(RestReceiver, self).__init__()
        self.log = logger.getLogger('Service')
        self.taskRunner = taskRunner
        self.receiveMethod = receiveMethod
        self.route('/nes/queue_reread', method='PUT', callback=self.__queueReread)
        self.route('/nes/queue_load', method='PUT', callback=self.__load)
        self.route('/nes/forgot_bad', method='PUT', callback=self.__forgotBad)

    def daemonize(self, host, port):
        self.log.debug('runing REST service on %s:%s...' % (host, port))
        t = threading.Thread(target=self.run, kwargs={'host': host, 'port': port})
        t.daemon = True
        t.start()

    def __queueReread(self):
        response.content_type = 'application/json'
        result = {u'success': False}
        try:
            QueueReceiver.timeShopsReread = 0
            self.log.info(u'Change timeShopsReread = %s' % QueueReceiver.timeShopsReread)
            result = {u'success': True}
        except Exception as e:
            self.log.warn(u'Ошибка REST сервиса. %s' % str(e))
            self.log.warn(traceback.format_exc())
            response.status = INTERNAL_SERVER_ERROR
        return json.dumps(result, ensure_ascii=False)

    def __load(self):
        body = json.loads(request.body.read())
        shopId = str(body.get('shopId'))
        response.content_type = 'application/json'
        result = {u'success': False}
        try:
            self.log.info('Put new task for load shop:%s' % shopId)
            self.taskRunner.addAsyncTask(shopId, self.receiveMethod, [shopId])
            result = {u'success': True}
        except:
            self.log.warn(traceback.format_exc())
            response.status = INTERNAL_SERVER_ERROR
        return json.dumps(result, ensure_ascii=False)

    def __forgotBad(self):
        body = json.loads(request.body.read())
        shopId = body.get('shopId')
        response.content_type = 'application/json'
        result = {u'success': False}
        try:
            self.log.info('Forgot bad task for shop:%s' % shopId)
            self.taskRunner.forgotBadTasks()
            result = {u'success': True}
        except Exception as e:
            self.log.warn(u'Ошибка REST сервиса. %s' % str(e))
            self.log.warn(traceback.format_exc())
            response.status = INTERNAL_SERVER_ERROR
        return json.dumps(result, ensure_ascii=False)


class QueueReceiver(object):
    timeShopsReread = 0

    def __init__(self, isService=True, isCash=True):
        config.setMode(isCash)
        self.log = logger.getLogger('Service')
        self.isService = isService
        self.serviceLock = servicelock.ServiceLock(
                os.path.normpath('%s/locks/lockQueueGetter' % tempfile.gettempdir()), self.log)

        signal.signal(signal.SIGTERM, self.__stop)
        atexit.register(self.__unlock)

        Transaction.fileQueueRoot = config.getString('FILEQUEUE', 'fileQueueRoot',
                '/linuxcash/cash/data/queues' if config.isCash else '/opt/queues')
        sendDictGenerateEventOnCS = config.getBool('MAIN', 'sendDictGenerateEventOnCS', True)
        Transaction.defaultNesGenerateCommandMode = 'increment' if sendDictGenerateEventOnCS else 'none'
        Transaction.defaultStatusServer = config.getString('CS', 'defaultStatusServer', 'localhost:8080')

        self.taskRunner = BasicTaskRunner() if config.isCash else MprocTaskRunner()
        self.taskRunner.init(config.getInt('MAIN', 'poolProcessSize', 1))

        self.log.debug(u'Режим сервиса = %s' % self.isService)
        self.log.debug(u'Запуск на кассе = %s' % config.isCash)
        if self.isService:
            self.serviceLock.lock(exitOnLock=True)
            if not config.isCash:
                self.__startRestServiceForCS()

    def start(self):
        self.log.info(u'Старт NES на %s %s' % (u'кассе' if config.isCash else u'КС', u'в режиме сервиса' if self.isService else ''))
        shopDests = []
        while True:
            if config.isCash:
                shopDests = ['cash']
            else:
                if QueueReceiver.timeShopsReread == 0 or (time.time() - 600) > QueueReceiver.timeShopsReread:
                    QueueReceiver.timeShopsReread = time.time()
                    self.log.debug('Получение списка магазинов')
                    tmpShopDests = FileQueueManager(queueRoot=Transaction.fileQueueRoot, log=self.log, location='cs').getShopDests(
                            prefixQueueName)
                    if tmpShopDests:
                        shopDests = tmpShopDests
                    self.log.debug('Список магазинов = %s' % shopDests)

            for shopDest in shopDests:
                self.taskRunner.addAsyncTask(shopDest, receiveMessages, [shopDest])

            if self.isService:
                time.sleep(20)
            else:
                break

    def __stop(self, signum, frame):
        self.log.info(u'Останов очереди')
        if self.taskRunner:
            self.taskRunner.stopAllTasks()
        self.log.info(u'Выход')
        exit(0)

    def __unlock(self):
        self.serviceLock.unlock()

    def __startRestServiceForCS(self):
        restServiceHost = config.getString('REST_SERVICE', 'host', '0.0.0.0')
        restServicePort = config.getInt('REST_SERVICE', 'port', 28081)
        self.restReceiver = RestReceiver(self.taskRunner, receiveMessages)
        try:
            self.restReceiver.daemonize(restServiceHost, restServicePort)
        except Exception as e:
            self.log.warn(u'Ошибка запуска REST сервиса: %s' % str(e))
            self.log.warn(traceback.format_exc())


if __name__ == '__main__':
    parser = optparse.OptionParser()
    parser.add_option("-s", "--service", action="store_true", dest="isService", help=u'Старт сервиса', default=False)
    parser.add_option("-t", "--type", dest="type", help=u'Тип сервиса(cash или cs)', default="cash")
    (options, args) = parser.parse_args(sys.argv)
    QueueReceiver(options.isService, options.type == "cash").start()
